//@flow
/**
 * @author Sven Koelpin
 */

import React, { PureComponent } from 'react';
import { render } from 'react-dom';
import './main.css';
import * as Server from './api/Server';
import type { AppSettings, Chat } from './domain/Entities';
import { User } from './domain/Entities';
import { Header } from './menu/Header';
import { UserList } from './user/UserList';
import NewsFeed from './feed/NewsFeed';
import ChatArea from './chat/ChatArea';


type State = {
    users: Array<User>;
    chats: Array<Chat>;
    appSettings: ?AppSettings;
}


class ReactChatApp extends PureComponent<void, any, State> {
    state: State;

    constructor() {
        super();
        (this: any).onMarkChatAsRead = this.onMarkChatAsRead.bind(this);
        this.state = {
            users: [],
            chats: [],
            appSettings: null
        };
    }

    componentDidMount() {
        this.initApp();
    }

    componentWillUnmount() {
        Server.unsubscribeAll();
    }

    async initApp() {
        await this.fetchAppSettings();
        await this.fetchUsers();
        this.listenToUserEvents();
        this.listenToChatEvents();
    }

    async fetchAppSettings() {
        const appSettings = (await Server.get('users/1/settings'): any);
        this.setState({appSettings});
    }

    async fetchUsers() {
        const users = (await Server.get('users'): any);
        this.setState({users});
    }

    listenToUserEvents() {
        Server.subscribeToUserChanges(changedUser => {
            this.setState({
                users: this.state.users.map(user => {
                    if (changedUser.id === user.id) {
                        return changedUser;
                    }
                    return user;
                }),
                chats: this.state.chats.map(chat => {
                    if (chat.user.id === changedUser.id) {
                        return Object.assign(({}: any), chat, {
                            user: changedUser
                        });
                    }
                    return chat;
                })
            });
        })
    }

    listenToChatEvents() {
        Server.subscribeToChatMessages(nextMessage => {
            const chatForMessage = this.state.chats.find(chat => chat.id === nextMessage.user);
            if (!chatForMessage) {
                this.setState({
                    chats: [...this.state.chats, {
                        id: nextMessage.user,
                        user: (this.state.users.find(user => user.id === nextMessage.user): any),
                        messages: [nextMessage],
                        unread: true
                    }]
                });
            } else {
                this.setState({
                    chats: this.state.chats.map(chat => {
                        if (chat.id === chatForMessage.id) {
                            return Object.assign(({}: any), chatForMessage, {messages: [...chatForMessage.messages, nextMessage], unread: true})
                        }
                        return chat;
                    })
                });
            }
        });
    }

    onMarkChatAsRead(chatToMark: Chat) {
        this.setState({
            chats: this.state.chats.map(chat => {
                if (chatToMark === chat) {
                    return Object.assign(({}: any), chat, {unread: false});
                }
                return chat;
            })
        });
    }


    render() {
        const {chats, users, appSettings} = this.state;
        const numUnread = chats.filter(chat => chat.unread).length;
        return (
            <section className='main'>
                <Header unreadChats={numUnread} lang={appSettings ? appSettings.lang : null}/>
                <div className='page'>
                    <div className='content'>
                        <NewsFeed/>
                        <ChatArea chats={chats} onMarkChatAsRead={this.onMarkChatAsRead}/>
                    </div>
                    <UserList users={users}/>
                </div>
            </section>
        )
    }
}


window.Perf = require('react-addons-perf');
window.addEventListener('DOMContentLoaded', () => {
    render(<ReactChatApp/>, document.getElementById('app'));
});